package com.docus.server.visitor;

import com.docus.core.util.Func;
import com.docus.core.util.json.JSON;
import com.docus.log.context.TrackContext;
import com.docus.log.processor.AbstractProcessor;
import com.docus.server.collect.infrastructure.dao.CollectTypeEnum;
import com.docus.server.collect.infrastructure.dao.StateEnum;
import com.docus.server.collect.infrastructure.enums.IIntegerEnum;
import com.docus.server.tool.SpringUtils;
import com.docus.server.ws.ITaskOriginalMessageService;
import com.docus.server.ws.IWsResult;
import com.docus.server.ws.convert.IConverter;
import com.fasterxml.jackson.core.type.TypeReference;
import lombok.extern.slf4j.Slf4j;

import java.util.Map;

/**
 * @author linruifeng
 */
@Slf4j
public class VisitorProcessor extends AbstractProcessor {
    private ITaskOriginalMessageService messageService;
    private IConverter converter;
    private IWsResult wsResult;

    /**
     * 前置通知
     */
    @Override
    public Object beforeProcess(TrackContext context) {
        super.beforeProcess(context);
        initBeans(context.getBeanNames());
        String message = (String) context.getArgs()[0];
        if (Func.isEmpty(message)) {
            throw new RuntimeException("参数为空");
        }
        String jsonStr = JSON.toJSON(converter.convert(message, context.getMethodName()));
        Long taskId = messageService.insertTaskOriginalMessage(jsonStr, message, IIntegerEnum.fromDisplay(CollectTypeEnum.class, context.getGroup()));
        Map<String, Object> params = context.getParams();
        params.put("taskId", taskId);
        params.put("jsonStr", jsonStr);
        return null;
    }

    /**
     * 后置和异常通知
     */
    @Override
    public Object doProcess(TrackContext context) {
        Map<String, Object> params = context.getParams();
        try {
            Long taskId = (Long) params.get("taskId");
            String afterReturnResult = (String) params.get("jsonStr");
            params.put("msg", context.getExMessageResult());

            if (Func.isNotEmpty(afterReturnResult)) {
                params.putAll(JSON.fromJSONWithGeneric(afterReturnResult, new TypeReference<Map<? extends String, ?>>() {
                }));
            }

            if (!context.isError()) {
                log.info("=== AOP 后置通知 ===");
                params.put("msg", "操作成功！");
                return wsResult.ok(params);
            } else {
                log.info("=== AOP 异常通知 ===");
                messageService.updateTaskOriginalMessage(taskId, afterReturnResult, StateEnum.FAIL);
                return wsResult.fail(params);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return wsResult.fail(params);
        }
    }

    /**
     * 初始化bean
     */
    private void initBeans(String[] beanNames) {
        this.converter = (IConverter) SpringUtils.getBean(beanNames[0]);
        this.wsResult = (IWsResult) SpringUtils.getBean(beanNames[1]);
        this.messageService = SpringUtils.getBean(ITaskOriginalMessageService.class);
    }
}
