package com.docus.server.ws.impl;

import com.docus.core.util.json.JSON;
import com.docus.log.annotation.TrackGroup;
import com.docus.log.context.TrackHelper;
import com.docus.server.archivefile.pojo.dto.ReportDTO;
import com.docus.server.visitor.VisitorProcessor;
import com.docus.server.ws.IReportServer;
import com.docus.server.ws.IReportService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

/**
 * @author wyb
 * @date 2023年3月3日13:42:36
 * @see com.docus.server.collect.infrastructure.dao.CollectTypeEnum 枚举
 * @see com.docus.server.ws.convert.IConverter 通用转化器
 * @see com.docus.server.ws.IWsResult 通用返回结果
 * @see VisitorProcessor 通用处理器
 */
@Service
@Slf4j
public class ReportServerImpl implements IReportServer {
    private static final String CONVERTER = "reportConverter";
    private static final String WS_RESULT = "wsReportResultImpl";
    @Resource
    private IReportService reportService;

    @Override
    @TrackGroup(
        group = "WEBSERVICE_XML_SA_REPORT",
        beanNames = {CONVERTER, WS_RESULT},
        processor = VisitorProcessor.class)
    public String pushSAReport(String message) {
        report();
        return null;
    }

    @Override
    @TrackGroup(
        group = "WEBSERVICE_XML_ICU_REPORT",
        beanNames = {CONVERTER, WS_RESULT},
        processor = VisitorProcessor.class)
    public String pushICUReport(String message) {
        log.info("收到重症消息：{}", message);
        report();
        return null;
    }

    @Override
    @TrackGroup(
        group = "WEBSERVICE_XML_EXAMINATION_REPORT",
        beanNames = {CONVERTER, WS_RESULT},
        processor = VisitorProcessor.class)
    public String pushExaminationReport(String message) {
        log.info("收到检验报告消息：{}", message);
        report();
        return null;
    }

    @Override
    @TrackGroup(
        group = "WEBSERVICE_XML_INSERT_INSPECTION_REPORT",
        beanNames = {CONVERTER, WS_RESULT},
        processor = VisitorProcessor.class)
    public String pushAddInspectionReport(String message) {
        log.info("收到检查报告新增消息：{}", message);
        report();
        return null;
    }

    @Override
    @TrackGroup(
        group = "WEBSERVICE_XML_UPDATE_INSPECTION_REPORT",
        beanNames = {CONVERTER, WS_RESULT},
        processor = VisitorProcessor.class)
    public String pushUpdateInspectionReport(String message) {
        log.info("收到检查报告更新消息：{}", message);
        report();
        return null;
    }

    private void report() {
        reportService.report(JSON.fromJSON((String) TrackHelper.getValue("jsonStr"), ReportDTO.class));
    }
}
